/*
 * $Id: NixieClock.h,v 1.16 2008/05/08 09:51:28 fvecoven Exp $
 *
 * Copyright (C) 2008 Frederic Vecoven
 *
 * This file is part of NixieClock
 *
 * NixieClock is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * NixieClock is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _NIXIE_CLOCK_H
#define	_NIXIE_CLOCK_H

/* flags */
typedef union {
	struct {
		unsigned do_tube :1;
		unsigned do_dcf :1;
		unsigned do_ad :1;
		unsigned do_command :1;
		unsigned do_alarm :1;
		unsigned temp_brightness :1;
		unsigned temp_date :1;
		unsigned do_save_tube_time :1;
		unsigned do_minute_work :1;
		unsigned do_hour_work :1;
	};
	unsigned int value;
} flags_t;

typedef union {
	struct {
		unsigned auto_brightness :1;
		unsigned date_format :1;
		unsigned alarm_enable :1;
		unsigned tube_save :1;
		unsigned tube_save_dcf :1;
		unsigned hour_leading_zero :1;
	};
	unsigned char value;
} prefs_t;


/* clock is running at 8MHz x 4 = 32MHz */
#define	CLOCK	32000000

/* timer 0 : 50uS */
#define	TMR0_H_RELOAD	0xFD
#define	TMR0_L_RELOAD	0xC0

/* timer 1 : 4096Hz clock. period 1/256s */
#define	TMR1_H_RELOAD	0xFF
#define	TMR1_L_RELOAD	0xF0


/* mode */
#define	MODE_TIME	0
#define	MODE_DCF	1
#define	MODE_AUTO_DCF	2
#define	MODE_EDITOR	3
#define	MODE_OFF	4
#define	MODE_AUTO_OFF	5
#define	MODE_PREVENT	6
#define	MODE_AUTO	7


/* address of preferences in eeprom */
#define	ADR_MAGIC		0x00
#define	ADR_PREFS		0x01
#define	ADR_BRIGHTNESS		0x02
#define	ADR_ALARM_HH		0x03
#define	ADR_ALARM_MM		0x04
#define	ADR_ALARM_DURATION	0x05
#define	ADR_TUBE_OFF_HH		0x06
#define	ADR_TUBE_OFF_MM		0x07
#define	ADR_TUBE_ON_HH		0x08
#define	ADR_TUBE_ON_MM		0x09
#define	ADR_DATE_DURATION	0x0a
#define	ADR_DATE_WHEN		0x0b
#define	ADR_PREVENT_RELOAD	0x0c
#define	ADR_PREVENT_HH		0x0d
#define	ADR_PREVENT_MM		0x0e

#define	PREFS_MAGIC		'x'



/* PORTA */
#define	ALARM_PIN		PORTAbits.RA0
#define	ALARM_TRIS		TRISAbits.TRISA0
#define	ALARM_ANSEL		ANSEL0bits.ANS0
#define	LED_RED_TRIS		TRISAbits.TRISA1
#define	LED_RED_ANSEL		ANSEL0bits.ANS1
#define	LED_RED_PIN		LATAbits.LATA1
#define	DP2			LED_RED_PIN
#define	LED_GREEN_TRIS		TRISAbits.TRISA2
#define	LED_GREEN_ANSEL		ANSEL0bits.ANS2
#define	LED_GREEN_PIN		LATAbits.LATA2
#define	DP1			LED_GREEN_PIN
#define	DCF_TRIS		TRISAbits.TRISA3
#define	DCF_ANSEL		ANSEL0bits.ANS3
#define	DCF_PIN			PORTAbits.RA3
#define	LDR_TRIS		TRISAbits.TRISA4
#define	LDR_ANSEL		ANSEL0bits.ANS4


/* PORTB : all outputs to tube digits */

/* PORTC

 0 : in  : 32kHz from DS1307
 1 : out : control high voltage (1 = on)
 2 : out : control anode 1
 3 : in  : IR
 4 : in/out : SDA
 5 : in/out : SCL
 6 : out : control anode 0
 7 : out : control anode 2

*/

#define	HV_TRIS		TRISCbits.TRISC1
#define	HV_PIN		LATCbits.LATC1
#define	HV_ON		HV_PIN = 1;
#define	HV_OFF		HV_PIN = 0;

#define	IR_SIGNAL	PORTCbits.RC3
#define	IR_TRIS		TRISCbits.TRISC3

#define	ANODE0		LATCbits.LATC6
#define	ANODE0_TRIS	TRISCbits.TRISC6
#define	ANODE1		LATCbits.LATC2
#define	ANODE1_TRIS	TRISCbits.TRISC2
#define	ANODE2		LATCbits.LATC7
#define	ANODE2_TRIS	TRISCbits.TRISC7


/* prototypes */
void set_mode(char);


/* variables */
extern volatile flags_t flags;
extern volatile prefs_t prefs;
extern volatile unsigned char alarm_duration;
extern volatile unsigned char show_date_when;
extern volatile unsigned char show_date_duration;
extern volatile unsigned char mode;
extern volatile int temp_show;


#endif
