/*
 * $Id: command.c,v 1.17 2008/05/08 09:51:28 fvecoven Exp $
 *
 * Copyright (C) 2008 Frederic Vecoven
 *
 * This file is part of NixieClock
 *
 * NixieClock is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * NixieClock is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <p18cxxx.h>
#include <delays.h>
#include "NixieClock.h"
#include "command.h"
#include "ds1307.h"
#include "dcf77.h"
#include "tubes.h"
#include "nec.h"
#include "sw_i2c.h"
#include "editor.h"


/* global variables */
volatile unsigned char beep = 0;


void handle_command(void)
{
	/*
	 * if alarm is currently going, assume that the user wants to
	 *  stop it !
	 */
	if (flags.do_alarm) {
		stop_alarm();
		return;
	}


	/*
	 * Handle the power key.
	 */
	if (command == KEY_POWER) {
		if (mode == MODE_OFF || mode == MODE_AUTO_OFF) {
			set_mode(MODE_TIME);
		} else {
			set_mode(MODE_OFF);
		}
		return;
	}


	/* POWEROFF mode */
	if (mode == MODE_OFF || mode == MODE_AUTO_OFF) {
		/* don't beep if keys are pressed */
		return;
	}


	/* TIME mode */
	if (mode == MODE_TIME) {

		switch (command) {

#if 0
		case KEY_TV_FM:
			set_mode(MODE_PREVENT);
			return;
#endif

		case KEY_FULL:
			set_mode(MODE_EDITOR);
			return;

		case KEY_CH_SURF:
			set_mode(MODE_DCF);
			return;

		case KEY_CH_UP:
			tubes_adjust_brightness(1);
			return;

		case KEY_CH_DOWN:
			tubes_adjust_brightness(-1);
			return;

		default:
			/* all other keys : show date */
			flags.temp_date = 1;
			temp_show = 512;	// 2 sec
			time_refresh();
			return;
		}
	}


	/* DCF mode */
	if (mode == MODE_DCF) {

		switch (command) {

		case KEY_CH_SURF:
			set_mode(MODE_TIME);
			return;

		}
	}


	/* cathode poisoning preventation */
	if (mode == MODE_PREVENT) {
		/* any command goes back to mode time */
		set_mode(MODE_TIME);
		return;
	}


	/* EDITOR mode */
	if (mode == MODE_EDITOR) {

		switch (command) {

		case KEY_FULL:
			/* exit editor mode */
			editor_timeout();
			return;

		case KEY_CH_UP:
			editor_up_down(1);
			return;

		case KEY_CH_DOWN:
			editor_up_down(-1);
			return;

		case KEY_ENTER:
			editor_enter();
			return;

		case KEY_VOL_UP:
			editor_inc_digit(1);
			return;

		case KEY_VOL_DOWN:
			editor_inc_digit(-1);
			return;

		case KEY_RECALL:
			editor_get();
			editor_refresh();
			return;

		case KEY_0:
			editor_num(0, 1);
			return;

		case KEY_1:
			editor_num(1, 1);
			return;

		case KEY_2:
			editor_num(2, 1);
			return;

		case KEY_3:
			editor_num(3, 1);
			return;

		case KEY_4:
			editor_num(4, 1);
			return;

		case KEY_5:
			editor_num(5, 1);
			return;

		case KEY_6:
			editor_num(6, 1);
			return;

		case KEY_7:
			editor_num(7, 1);
			return;

		case KEY_8:
			editor_num(8, 1);
			return;

		case KEY_9:
			editor_num(9, 1);
			return;

		}
	}

	/* invalid command */
	SINGLE_BEEP;
}

