/*
 * $Id: nec.c,v 1.3 2008/05/11 08:38:50 fvecoven Exp $
 *
 * Copyright (C) 2008 Frederic Vecoven
 *
 * This file is part of NixieClock
 *
 * NixieClock is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * NixieClock is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <p18cxxx.h>
#include "NixieClock.h"
#include "nec.h"



/* global variables */
volatile nec_t nec;
volatile unsigned long nec_data;
volatile unsigned char nec_bitcount;
volatile unsigned char IR_count;
volatile unsigned char command;


/* prototypes */
static void shift_nec_data(unsigned char);



#pragma code

void nec_enable(void)
{
	INTCONbits.INT0IF = 0;
	INTCONbits.INT0IE = 1;
	nec_reset();
}


void nec_reset(void)
{
	INTCON2bits.INTEDG0 = 0;
	nec.first = 1;
}


void handle_nec_interrupt(void)
{
	/* reverse trigger */
	INTCON2bits.INTEDG0 = !INTCON2bits.INTEDG0;

	/* check if we have the first negative edge */
	if (nec.first) {
		IR_count = 0;
		nec.first = 0;
		nec.second = 1;
		return;
	}

	/* check for timeout */
	if (IR_count > NEC_TIMEOUT) {
		nec_reset();
		return;
	}

	/* 2nd edge. We are looking for a 8.4ms pulse */
	if (nec.second) {
		if (IR_count > NEC_START) {
			nec.second = 0;
			nec.third = 1;
			IR_count = 0;
		} else {
			/* pulse too short */
			nec_reset();
		}
		return;
	}

	/* 3rd edge. We are looking for a 4.2ms pulse */
	if (nec.third) {
		if (IR_count > NEC_START2) {
			nec.third = 0;
			nec.in_pulse = 1;
			IR_count = 0;
			nec_data = 0;
			nec_bitcount = 0;
		} else {
			/* pulse too short */
			nec_reset();
		}
		return;
	}

	/* we are expecting a pulse of 526us */
	if (nec.in_pulse) {
		if (IR_count < NEC_PULSE_MIN || IR_count > NEC_PULSE_MAX) {
			nec_reset();
			return;
		}

		nec.in_pulse = 0;
		nec.in_0_or_1 = 1;
		IR_count = 0;
		return;
	}

	/* we are at the begin of the next pulse */
	if (nec.in_0_or_1) {
		if (IR_count > NEC_1) {
			nec_reset();
			return;
		}

		if (IR_count < NEC_0) {
			shift_nec_data(0);
		} else {
			shift_nec_data(1);
		}

		nec.in_pulse = 1;
		IR_count = 0;
	}
}





static void shift_nec_data(unsigned char value)
{
	nec_data = nec_data << 1;
	nec_data |= value;

	nec_bitcount++;
	if (nec_bitcount == 32) {
		unsigned char x1;
		unsigned char x2;

		/* format is XXxxYYyy  where xx = ~XX */
		x1 = (unsigned char)((nec_data & 0xFF000000) >> 24);
		x2 = (unsigned char)((nec_data & 0x00FF0000) >> 16);
		if (x1 == ~x2 && x2 == NEC_ADDRESS) {
			x1 = (unsigned char)((nec_data & 0xFF00) >> 8);
			x2 = (unsigned char)(nec_data & 0x00FF);
			if (x1 == ~x2) {
				command = x2;
				flags.do_command = 1;
			}
		}
		nec_reset();
	}
}

