/*
 * $Id: sw_i2c.c,v 1.4 2008/05/08 09:51:28 fvecoven Exp $
 *
 * Copyright (C) 2008 Frederic Vecoven
 *
 * This file is part of NixieClock
 *
 * NixieClock is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * NixieClock is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <p18cxxx.h>
#include <delays.h>
#include "sw_i2c.h"

/* globals */
unsigned char I2C_BUFFER;
unsigned char BIT_COUNTER;


void i2c_init(void)
{
	DATA_HI;
	CLOCK_HI;
}


void i2c_start(void)
{
	DATA_LAT = 0;
	DATA_LOW;
	Delay10TCYx(4);
}


void i2c_stop(void)
{
	SCLK_LAT = 0;
	CLOCK_LOW;
	DATA_LAT = 0;
	DATA_LOW;
	Delay10TCYx(4);
	CLOCK_HI;
	Delay10TCYx(4);
	DATA_HI;
	Nop();
	Nop();
}


void i2c_restart(void)
{
	SCLK_LAT = 0;
	CLOCK_LOW;
	DATA_HI;
	Delay10TCYx(4);
	CLOCK_HI;
	Delay10TCYx(4);
	DATA_LAT = 0;
	DATA_LOW;
	Delay10TCYx(4);
}


signed char i2c_ack(void)
{
	SCLK_LAT = 0;
	CLOCK_LOW;
	DATA_HI;
	Delay10TCYx(4);
	CLOCK_HI;
	Delay10TCYx(20);

	if (DATA_PIN) {
		return (-1); 	// slave didn't ack
	} else {
		return (0);
	}
}


signed char i2c_clock_test(void)
{
	Delay10TCYx(12);	// wait period
	if (!SCLK_PIN) {
		return (-2);	// clock still low after wait
	} else {
		return (0);
	}
}


unsigned int i2c_read(void)
{
	BIT_COUNTER = 8;
	SCLK_LAT = 0;
	do {
		CLOCK_LOW;
		DATA_HI;
		Delay10TCYx(4);
		CLOCK_HI;
		Delay10TCY();

		if (!SCLK_PIN) {
			if (i2c_clock_test()) {
				return (-2);
			}
		}

		I2C_BUFFER <<= 1;
		I2C_BUFFER &= 0xFE;

		if (DATA_PIN) {
			I2C_BUFFER |= 0x01;
		}

	} while (--BIT_COUNTER);

	return ((unsigned int) I2C_BUFFER);
}


unsigned int i2c_getc(void)
{
	unsigned int x;

	x = i2c_read();
	if (x & 0xFF00) {
		return (-1);
	}
	if (!i2c_ack()) {
		return (-1);
	}
	return (x);
}


signed char i2c_gets(unsigned char *ptr, unsigned char length)
{
	unsigned int thold;

	while (length--) {
		thold = i2c_read();
		if (thold & 0xFF00) {
			return (-1);
		} else {
			*ptr++ = thold;
		}

		if (!length) {
			CLOCK_LOW;
			DATA_HI;
			Delay10TCYx(4);
			CLOCK_HI;
			Delay10TCYx(4);
		} else {
			CLOCK_LOW;
			DATA_LAT = 0;
			DATA_LOW;
			Delay10TCYx(4);
			CLOCK_HI;
			Delay10TCYx(4);
		}
	}

	return (0);
}



signed char i2c_write(unsigned char data)
{
	BIT_COUNTER = 8;
	I2C_BUFFER = data;
	SCLK_LAT = 0;

	do {
		if (!SCLK_PIN) {
			if (i2c_clock_test()) {
				return (-1);
			}
		} else {
			I2C_BUFFER &= 0xFF;
			_asm
			rlcf I2C_BUFFER,1,1
			_endasm

			if (STATUS & 0x01) {
				CLOCK_LOW;
				DATA_HI;
				Delay10TCYx(4);
				CLOCK_HI;
				Delay10TCYx(4);
			} else {
				CLOCK_LOW;
				DATA_LAT = 0;
				DATA_LOW;
				Delay10TCYx(4);
				CLOCK_HI;
				Delay10TCYx(4);
			}

			BIT_COUNTER--;
		}
	} while (BIT_COUNTER);

	return (0);
}

signed char i2c_putc(unsigned char data)
{
	if (i2c_write(data)) {
		return (-1);
	}
	if (i2c_ack()) {
		return (-1);
	}
	return (0);
}


signed char i2c_puts(unsigned char *ptr)
{
	while (*ptr) {
		if (i2c_write(*ptr++)) {
			return (-1);
		} else {
			if (i2c_ack()) {
				return (-1);
			}
		}
	}

	return (0);
}


