/*
 * $Id: util.c,v 1.4 2008/05/08 09:51:29 fvecoven Exp $
 *
 * Copyright (C) 2008 Frederic Vecoven
 *
 * This file is part of NixieClock
 *
 * NixieClock is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * NixieClock is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <p18cxxx.h>
#include "util.h"


/*
 * un-bcd : 0x00 - 0x99 --> 0 - 99
 */
char unbcd(unsigned char x)
{
	char res = 0;
	char i;

	i = (x & 0xF0) >> 4;
	while (i > 0) {
		res += 10;
		i--;
	}
	res += (x & 0x0F);

	return (res);
}


/*
 * bcd : 0 - 99 -->  0x00 - 0x99
 */
unsigned char bcd(char x)
{
	unsigned char res = 0;

	while (x >= 10) {
		x -= 10;
		res++;
	}
	res <<= 4;
	res |= x;

	return (res);
}


/*
 * read one byte from EEPROM
 */
unsigned char ee_read(unsigned char address)
{
	EEADR = address;
	EECON1bits.EEPGD = 0;		// data memory
	EECON1bits.RD = 1;

	return (EEDATA);
}


/*
 * write one byte into EEPROM
 */
void ee_write(unsigned char address, unsigned char value)
{
	EEADR = address;
	EEDATA = value;
	EECON1bits.CFGS = 0;
	EECON1bits.EEPGD = 0;		// data memory
	EECON1bits.WREN = 1;
	INTCONbits.GIE = 0;
	_asm
		MOVLW	0x55
		MOVWF	EECON2, 0
		MOVLW	0xAA
		MOVWF	EECON2, 0
	_endasm
	EECON1bits.WR = 1;

	INTCONbits.GIE = 1;

	while (EECON1bits.WR == 1);		// wait until write cycle is done

	EECON1bits.WREN = 0;
}
